
local f = CreateFrame("Frame", "CharDeathCounterTimer", UIParent)
local panel = CreateFrame("Frame")

local CharDeathCounterFont = "Fonts\\ARIALN.ttf"

local xpos1, ypos1, xpos2, ypos2, xpos3, ypos3 = 0
local _, englishClass, _ = UnitClass("player")
local _, _, _, classColor = GetClassColor(englishClass)
local defaultColor = "ffff5050"
local skippedColor = "ff00ffff"
local NAME, S = ...

SLASH_CharDeathCounter1 = "/cdc"
SLASH_CharDeathCounter2 = "/CharDeathCounter"

SlashCmdList["CharDeathCounter"] = function(msg)
	if msg == "config" then
		InterfaceOptionsFrame_OpenToCategory(panel);
		InterfaceOptionsFrame_OpenToCategory(panel);
	elseif msg == "reset" then
		ResetStats()
		UpdateDisplayedText()
	elseif msg == "skip" then
		SkipPreviousDeath()
	else
		print("Unknown command, command list:")
		print("|cff00ffff/cdc config|r: Open config panel")
		print("|cff00ffff/cdc skip|r: Skip previous death")
		print("|cff00ffff/cdc reset|r: Reset all stats for current character")
	end
end

function InitialiseCDCFrame()
	f:SetWidth(1) 
	f:SetHeight(1) 
	f:SetAlpha(1)
	f:RegisterForDrag("LeftButton")

	f.text = f:CreateFontString(nil,"ARTWORK") 
	f.text:SetPoint("TOPLEFT",0,0)
	f.text:SetJustifyH("LEFT")
	f.text:SetJustifyV("TOP")

	f.timer = f:CreateFontString(nil,"ARTWORK") 
	f.timer:SetPoint("TOPRIGHT",f,"TOPRIGHT",0,0)
	f.timer:SetJustifyH("RIGHT")
	f.timer:SetJustifyV("TOP")

	f.counter = f:CreateFontString(nil,"ARTWORK") 
	f.counter:SetPoint("TOPRIGHT",f.timer,"TOPLEFT",-10,0)
	f.counter:SetJustifyH("CENTER")
	f.counter:SetJustifyV("TOP")

	f.skippedText = f:CreateFontString(nil,"ARTWORK") 
	f.skippedText:SetPoint("BOTTOMLEFT",f,"BOTTOMLEFT",0,0)
	f.skippedText:SetJustifyH("LEFT")
	f.skippedText:SetJustifyV("TOP")

	f.skippedTimer = f:CreateFontString(nil,"ARTWORK") 
	f.skippedTimer:SetPoint("BOTTOMRIGHT",f,"BOTTOMRIGHT",0,0)
	f.skippedTimer:SetJustifyH("RIGHT")
	f.skippedTimer:SetJustifyV("TOP")

	f.skippedCounter = f:CreateFontString(nil,"ARTWORK") 
	f.skippedCounter:SetPoint("TOPRIGHT",f.skippedTimer,"TOPLEFT",-10,0)
	f.skippedCounter:SetJustifyH("CENTER")
	f.skippedCounter:SetJustifyV("TOP")


	function f:OnEvent(event, arg1, arg2)
		if (event == "ADDON_LOADED" and arg1 == NAME) then
			AddonLoadedEventHandler()
			UpdateSkipButton()

		elseif (event == "PLAYER_LOGIN") then
			InitStats()
			CheckIfNewChar()
			UpdateDisplayedText()
			UpdateSkipButton()

		elseif (event == "PLAYER_DEAD") then
			-- fire twice if reconnecting dead ( ??? )
			if not CharDeathCounter["isDead"] then
				CharDeathCounter["deathCount"] = CharDeathCounter["deathCount"] + 1
				CharDeathCounter["timeSpentDead"] = CharDeathCounter["timeSpentDead"] + CharDeathCounter["currentDeathTimer"]
				CharDeathCounter["currentDeathTimer"] = 0
			end
			CharDeathCounter["isDead"] = UnitIsDeadOrGhost("player")
			UpdateSkipButton()
			UpdateDisplayedText()

		elseif event == "PLAYER_ALIVE" or event == "PLAYER_UNGHOST" then
			CharDeathCounter["isDead"] = UnitIsDeadOrGhost("player")
			UpdateSkipButton()

		elseif (event == "TIME_PLAYED_MSG") then
			f:UnregisterEvent("TIME_PLAYED_MSG")
			if arg1 < 5 and CharDeathCounter ~= nil and UnitLevel("player") == 1 and UnitXP("player") == 0 and CharDeathCounter["deathCount"] > 0 then
				ResetStats()
				UpdateDisplayedText()
			end
		end
	end

	function f:OnUpdate(arg1)
		if CharDeathCounter["isDead"] then
			CharDeathCounter["currentDeathTimer"] = CharDeathCounter["currentDeathTimer"] + arg1
			UpdateDisplayedText()
		end
	end

	f:RegisterEvent("ADDON_LOADED")
	f:RegisterEvent("PLAYER_DEAD")
	f:RegisterEvent("PLAYER_LOGIN")
	f:RegisterEvent("PLAYER_ALIVE")
	f:RegisterEvent("PLAYER_UNGHOST")
	f:RegisterEvent("TIME_PLAYED_MSG")

	f:SetScript("OnEvent", f.OnEvent)
end

function CharDeathCounterMovable()
	f:SetMovable(true)
	f:EnableMouse(true)
end

function AddonLoadedEventHandler()
	f:SetScript("OnMouseDown", function(self, button)
		if button == "LeftButton" and not self.isMoving and CharDeathCounterOptions["Move"] then
			_, _, _, xpos1, ypos1 = f:GetPoint(1)
			self:StartMoving();
			_, _, _, xpos2, ypos2 = f:GetPoint(1)
			self.isMoving = true;
		end
	end)
	f:SetScript("OnMouseUp", function(self, button)
		if button == "LeftButton" and self.isMoving and CharDeathCounterOptions["Move"] then
			_, _, _, xpos3, ypos3 = f:GetPoint(1)
			self:StopMovingOrSizing()
			self.isMoving = false;
			CharDeathCounterOptions["xpos"] = xpos1 + xpos3 - xpos2
			CharDeathCounterOptions["ypos"] = ypos1 + ypos3 - ypos2
			self:SetPoint("TOPLEFT",CharDeathCounterOptions["xpos"],CharDeathCounterOptions["ypos"])
		end
	end)
	f:SetScript("OnHide", function(self)
		if self.isMoving then
			self:StopMovingOrSizing();
			self.isMoving = false;
		end
	end)

	panel.FontSlider:SetScript("OnValueChanged", function(self, newvalue)
		newvalue = floor(newvalue+0.5)
		CharDeathCounterOptions["FontSize"] = newvalue
		_G["CharDeathCounterFontSliderText"]:SetText("Font Size ("..newvalue..")");
		panel.FontSlider:SetValue(newvalue)
		f.text:SetFont(CharDeathCounterFont, newvalue, "OUTLINE")
		f.counter:SetFont(CharDeathCounterFont, newvalue, "OUTLINE")
		f.timer:SetFont(CharDeathCounterFont, newvalue, "OUTLINE")
		f.skippedText:SetFont(CharDeathCounterFont, newvalue, "OUTLINE")
		f.skippedCounter:SetFont(CharDeathCounterFont, newvalue, "OUTLINE")
		f.skippedTimer:SetFont(CharDeathCounterFont, newvalue, "OUTLINE")
		UpdateDisplayedText()
	end)

	panel.ClassColorCheckButton:SetScript("OnClick",function()
		CharDeathCounterOptions["ClassColor"] = panel.ClassColorCheckButton:GetChecked()
		UpdateDisplayedText()
	end)

	panel.ShowHideCheckButton:SetScript("OnClick",function()
		if panel.ShowHideCheckButton:GetChecked() then
			f:Show()
			CharDeathCounterOptions["Show"] = true
		else
			CharDeathCounterOptions["Show"] = false
			f:Hide()
		end
	end)

	panel.MoveCheckButton:SetScript("OnClick",function()
		if panel.MoveCheckButton:GetChecked() then
			CharDeathCounterOptions["Move"] = true
			CharDeathCounterMovable()
		else
			CharDeathCounterOptions["Move"] = false
			f:SetMovable(false)
			f:EnableMouse(false)
		end
	end)
	
	panel.SkipCheckButton:SetScript("OnClick",function()
		if panel.SkipCheckButton:GetChecked() then
			CharDeathCounterOptions["Skip"] = true
			f.skippedText:Show()
			f.skippedCounter:Show()
			f.skippedTimer:Show()
		else
			CharDeathCounterOptions["Skip"] = false
			f.skippedText:Hide()
			f.skippedCounter:Hide()
			f.skippedTimer:Hide()
		end
		UpdateSkipButton()
		UpdateDisplayedText()
	end)

	InitStats()
	InitialiseOptions()
	UpdateDisplayedText()

	f:SetScript("OnUpdate", f.OnUpdate)
end

function UpdateSkipButton()
	if CharDeathCounterOptions["Skip"] and CharDeathCounter["currentDeathTimer"] > 0 and CharDeathCounter["deathCount"] > 0 and not CharDeathCounter["isDead"] then
		f.SkipButton:Show()
	else
		f.SkipButton:Hide()
	end
end

function InitialiseOptions()

	if CharDeathCounterOptions == nil then
		CharDeathCounterOptions = {
			xpos = 5,
			ypos = -120,
			FontSize = 13,
			Show = true,
			Move = f.isMoving,
			ClassColor = false,
			Skip = true
		}
	else
		if CharDeathCounterOptions["xpos"] == nil then
			CharDeathCounterOptions["xpos"] = 5
		end
		if CharDeathCounterOptions["ypos"] == nil then
			CharDeathCounterOptions["ypos"] = -120
		end
		if CharDeathCounterOptions["FontSize"] == nil then
			CharDeathCounterOptions["FontSize"] = 13
		end
		if CharDeathCounterOptions["Show"] == nil then
			CharDeathCounterOptions["Show"] = true
		end
		if CharDeathCounterOptions["ClassColor"] == nil then
			CharDeathCounterOptions["ClassColor"] = false
		end
		if CharDeathCounterOptions["Skip"] == nil then
			CharDeathCounterOptions["Skip"] = true
		end
		CharDeathCounterOptions["Move"] = f.isMoving
	end

	f:SetPoint("TOPLEFT", CharDeathCounterOptions["xpos"], CharDeathCounterOptions["ypos"])

	if CharDeathCounterOptions["Show"] then
		f:Show()
	else
		f:Hide()
	end

	if CharDeathCounterOptions["Skip"] then
		f.skippedText:Show()
		f.skippedCounter:Show()
		f.skippedTimer:Show()
	else
		f.skippedText:Hide()
		f.skippedCounter:Hide()
		f.skippedTimer:Hide()
	end

	SetFont(CharDeathCounterOptions["FontSize"])

	f.text:SetText("Death:")
	f.text:SetWidth(f.text:GetUnboundedStringWidth()+1)
	f.skippedText:SetText("Death skip:")
	f.skippedText:SetWidth(f.skippedText:GetUnboundedStringWidth()+1)
	
	f.SkipButton = CreateFrame("Button", "CharDeathCounterSkipButton", f, "UIPanelButtonTemplate");
	f.SkipButton:SetSize(50,22)
	_G["CharDeathCounterSkipButtonText"]:SetText("Skip");
	f.SkipButton:SetPoint("TOPLEFT", f, "BOTTOMLEFT", 0, -5);

	f.SkipButton:SetScript("OnClick", SkipPreviousDeath)

	panel.FontSlider:SetValue(CharDeathCounterOptions["FontSize"])
	panel.ClassColorCheckButton:SetChecked(CharDeathCounterOptions["ClassColor"])
	panel.ShowHideCheckButton:SetChecked(CharDeathCounterOptions["Show"])
	panel.MoveCheckButton:SetChecked(CharDeathCounterOptions["Move"])
	panel.SkipCheckButton:SetChecked(CharDeathCounterOptions["Skip"])

	_G["CharDeathCounterFontSliderText"]:SetText("Font Size ("..CharDeathCounterOptions["FontSize"]..")");
end

function InitialiseCDCPanel()
	panel.name = NAME
	InterfaceOptions_AddCategory(panel)

	panel.title = panel:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	panel.title:SetPoint("TOPLEFT", 16, -16)
	panel.title:SetText(NAME)

	panel.FontSlider = CreateFrame("Slider", "CharDeathCounterFontSlider", panel, "OptionsSliderTemplate")
	panel.FontSlider:SetOrientation('HORIZONTAL')
	panel.FontSlider.tooltipText = 'Size of the text displayed on screen for CharDeathCounter'
	_G["CharDeathCounterFontSliderLow"]:SetText("10");
	_G["CharDeathCounterFontSliderHigh"]:SetText("20");
	panel.FontSlider:SetPoint("TOPLEFT", panel.title, "BOTTOMLEFT", 0, -26)
	panel.FontSlider:SetMinMaxValues(10,20)
	panel.FontSlider:SetValueStep(1)
	panel.FontSlider:SetThumbTexture("Interface\\Buttons\\UI-SliderBar-Button-Horizontal")

	panel.ClassColorCheckButton = CreateFrame("CheckButton", "CharDeathCounterClassColorCheckButton", panel, "ChatConfigCheckButtonTemplate");
	panel.ClassColorCheckButton.tooltip = "Use class color for displayed counter and timer";
	_G["CharDeathCounterClassColorCheckButtonText"]:SetText("Use class color");
	panel.ClassColorCheckButton:SetPoint("TOPLEFT", panel.FontSlider, "BOTTOMLEFT", 0, -26);

	panel.ShowHideCheckButton = CreateFrame("CheckButton", "CharDeathCounterShowHideCheckButton", panel, "ChatConfigCheckButtonTemplate");
	panel.ShowHideCheckButton.tooltip = "Show/Hide CharDeathCounter";
	_G["CharDeathCounterShowHideCheckButtonText"]:SetText("Show CharDeathCounter");
	panel.ShowHideCheckButton:SetPoint("TOPLEFT", panel.ClassColorCheckButton, "BOTTOMLEFT", 0, 0);

	panel.MoveCheckButton = CreateFrame("CheckButton", "CharDeathCounterMoveCheckButton", panel, "ChatConfigCheckButtonTemplate");
	panel.MoveCheckButton.tooltip = "CharDeathCounter can be moved with the Interface Options window open";
	_G["CharDeathCounterMoveCheckButtonText"]:SetText("Move CharDeathCounter");
	panel.MoveCheckButton:SetPoint("TOPLEFT", panel.ShowHideCheckButton, "BOTTOMLEFT", 0, 0);

	panel.SkipCheckButton = CreateFrame("CheckButton", "CharDeathCounterSkipCheckButton", panel, "ChatConfigCheckButtonTemplate");
	panel.SkipCheckButton.tooltip = "Show/Hide Death Skip counter and timer";
	_G["CharDeathCounterSkipCheckButtonText"]:SetText("Show Death Skip");
	panel.SkipCheckButton:SetPoint("TOPLEFT", panel.MoveCheckButton, "BOTTOMLEFT", 0, 0);
	
	panel.default = function (self)
		InitialiseOptions()
		UpdateDisplayedText()
		print("[CharDeathCounter] Settings have been reset to default.")
	end
end

function GetColor()
	if CharDeathCounterOptions["ClassColor"] then
		return classColor
	end
	return defaultColor
end

function GetSkippedColor()
	if CharDeathCounterOptions["ClassColor"] then
		return classColor
	end
	return skippedColor
end

function GetFormatedTimer(seconds, color)
	local hours = "00"
	local mins = "00"
	local secs = "00"
	local ms = "0"

	if seconds > 0 then
		hours = string.format("%02.f", math.floor(seconds / 3600))
		mins = string.format("%02.f", math.floor(seconds / 60 - ( hours * 60)))
		secs = string.format("%02.f", math.floor(seconds % 60))
		ms = string.format("%01.f", math.floor((seconds * 10) % 10))
	end
	return string.format("|c%s%s:%s:%s.%s|r", color, hours, mins, secs, ms)
end

function GetFormatedCounter(counter, color)
	return string.format("|c%s%d|r", color, counter)
end

function UpdateDisplayedText()
	local counter = CharDeathCounter["deathCount"]
	local timer = CharDeathCounter["timeSpentDead"] + CharDeathCounter["currentDeathTimer"]
	if not CharDeathCounterOptions["Skip"] then
		counter = counter + CharDeathCounter["skippedDeathCount"]
		timer = timer + CharDeathCounter["skippedDeathTimer"]
	end
	
	f.counter:SetText(GetFormatedCounter(counter, GetColor()))
	f.timer:SetText(GetFormatedTimer(timer, GetColor()))
	
	f.skippedCounter:SetText(GetFormatedCounter(CharDeathCounter["skippedDeathCount"], GetSkippedColor()))
	f.skippedTimer:SetText(GetFormatedTimer(CharDeathCounter["skippedDeathTimer"], GetSkippedColor()))
	
	f.text:SetWidth(f.text:GetUnboundedStringWidth())
	f.text:SetHeight(f.text:GetStringHeight())
	
	
	f.skippedText:SetWidth(f.skippedText:GetUnboundedStringWidth())
	f.skippedText:SetHeight(f.skippedText:GetStringHeight())
	
	local counterWidth = 0
	if f.counter:GetUnboundedStringWidth() > f.skippedCounter:GetUnboundedStringWidth() then
		counterWidth = f.counter:GetUnboundedStringWidth()
	else
		counterWidth = f.skippedCounter:GetUnboundedStringWidth()
	end
	
	local timerWidth = 0
	if f.timer:GetUnboundedStringWidth() > f.skippedTimer:GetUnboundedStringWidth() then
		timerWidth = f.timer:GetUnboundedStringWidth()
	else
		timerWidth = f.skippedTimer:GetUnboundedStringWidth()
	end
	
	f.timer:SetWidth(timerWidth)
	f.skippedTimer:SetWidth(timerWidth)
	
	f.timer:SetHeight(f.timer:GetStringHeight())
	f.skippedTimer:SetHeight(f.skippedTimer:GetStringHeight())
	
	f.counter:SetWidth(counterWidth)
	f.skippedCounter:SetWidth(counterWidth)
	
	f.counter:SetHeight(f.counter:GetStringHeight())
	f.skippedCounter:SetHeight(f.skippedCounter:GetStringHeight())


	local firstLineWidth = f.text:GetUnboundedStringWidth() + f.timer:GetUnboundedStringWidth() + f.counter:GetUnboundedStringWidth()
	local secondLineWidth = f.skippedText:GetUnboundedStringWidth() + f.skippedTimer:GetUnboundedStringWidth() + f.skippedCounter:GetUnboundedStringWidth()

	if firstLineWidth > secondLineWidth then
		f:SetWidth(firstLineWidth + 20)
	else
		f:SetWidth(secondLineWidth + 20)
	end

	f:SetHeight(f.text:GetStringHeight() + f.skippedText:GetStringHeight() + 5)
end

function CheckIfNewChar()
	if CharDeathCounter ~= nil and UnitLevel("player") == 1 and UnitXP("player") == 0 and CharDeathCounter["deathCount"] > 0 then
		RequestTimePlayed()
	end
end

function InitStats()
	if CharDeathCounter == nil then
		CharDeathCounter = {
			deathCount = 0,
			timeSpentDead = 0,
			skippedDeathCount = 0,
			skippedDeathTimer = 0,
			currentDeathTimer = 0
		}
	end
	if CharDeathCounter["deathCount"] == nil then
		CharDeathCounter["deathCount"] = 0
	end
	if CharDeathCounter["timeSpentDead"] == nil then
		CharDeathCounter["timeSpentDead"] = 0
	end
	if CharDeathCounter["skippedDeathCount"] == nil then
		CharDeathCounter["skippedDeathCount"] = 0
	end
	if CharDeathCounter["skippedDeathTimer"] == nil then
		CharDeathCounter["skippedDeathTimer"] = 0
	end	
	if CharDeathCounter["currentDeathTimer"] == nil then
		CharDeathCounter["currentDeathTimer"] = 0
	end
	CharDeathCounter["isDead"] = UnitIsDeadOrGhost("player")
end

function ResetStats()
	CharDeathCounter = {
		deathCount = 0,
		skippedDeathCount = 0,
		timeSpentDead = 0,
		skippedDeathTimer = 0,
		currentDeathTimer = 0,
		isDead = UnitIsDeadOrGhost("player")
	}
end

function SkipPreviousDeath()
	if CharDeathCounter["isDead"] then
		print("|cff00ffffYou can't skip previous death while dead.|r")
	elseif CharDeathCounter["currentDeathTimer"] > 0 and CharDeathCounter["deathCount"] > 0 then
		CharDeathCounter["skippedDeathTimer"] = CharDeathCounter["skippedDeathTimer"] + CharDeathCounter["currentDeathTimer"]
		CharDeathCounter["skippedDeathCount"] = CharDeathCounter["skippedDeathCount"] + 1
		CharDeathCounter["deathCount"] = CharDeathCounter["deathCount"] - 1
		CharDeathCounter["currentDeathTimer"] = 0
		UpdateSkipButton()
		UpdateDisplayedText()
	else
		print("|cff00ffffNothing to skip.|r")
	end
end

function SetFont(value)
	f.text:SetFont(CharDeathCounterFont, value, "OUTLINE")
	f.counter:SetFont(CharDeathCounterFont, value, "OUTLINE")
	f.timer:SetFont(CharDeathCounterFont, value, "OUTLINE")
	f.skippedText:SetFont(CharDeathCounterFont, value, "OUTLINE")
	f.skippedCounter:SetFont(CharDeathCounterFont, value, "OUTLINE")
	f.skippedTimer:SetFont(CharDeathCounterFont, value, "OUTLINE")
end

InitialiseCDCFrame()
InitialiseCDCPanel()
